@@ -1,21 +1,17 @@
-# Copyright (C) 2007, 2008  Andrey Golovizin
-#
-# This file is part of pybtex.
-#
-# pybtex is free software; you can redistribute it and/or modify
-# under the terms of the GNU General Public License as published by the
-# Free Software Foundation; either version 2 of the License, or (at your
-# option) any later version.
-#
-# pybtex is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details.
-#
+# Copyright (C) 2007, 2008, 2009  Andrey Golovizin
+# 
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+# 
 # You should have received a copy of the GNU General Public License
-# along with pybtex; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
-# USA
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from pybtex.bibtex.exceptions import BibTeXError
 from pybtex.bibtex.builtins import builtins
@@ -176,7 +172,7 @@ class Interpreter(object):
 #        print 'pop <%s>' % value
         return value
 
-    def getToken(self):
+    def get_token(self):
         return self.bst_script.next()
 
     def add_variable(self, name, value):
@@ -187,10 +183,10 @@ class Interpreter(object):
     def output(self, string):
         self.output_file.write(string)
 
-    def run(self, bst_script, citations, bib_file, bbl_file):
+    def run(self, bst_script, citations, bib_files, bbl_file):
         self.bst_script = iter(bst_script)
         self.citations = citations
-        self.bib_file = bib_file
+        self.bib_files = bib_files
         self.output_file = bbl_file
 
         for i in self.bst_script:
@@ -203,36 +199,36 @@ class Interpreter(object):
         self.output_file.close()
 
     def command_entry(self):
-        for id in self.getToken():
+        for id in self.get_token():
             name = id.value()
             self.add_variable(name, Field(self, name))
         self.add_variable('crossref', Field(self, 'crossref'))
-        for id in self.getToken():
+        for id in self.get_token():
             name = id.value()
             self.add_variable(name, EntryInteger(self, name))
-        for id in self.getToken():
+        for id in self.get_token():
             name = id.value()
             self.add_variable(name, EntryString(self, name))
 
     def command_execute(self):
 #        print 'EXECUTE'
-        self.getToken()[0].execute(self)
+        self.get_token()[0].execute(self)
 
     def command_function(self):
-        name = self.getToken()[0].value()
-        body = self.getToken()
+        name = self.get_token()[0].value()
+        body = self.get_token()
         self.vars[name] = Function(body)
 
     def command_integers(self):
 #        print 'INTEGERS'
-        for id in self.getToken():
+        for id in self.get_token():
             self.vars[id.value()] = Integer()
 
     def command_iterate(self):
         self._iterate(self.citations)
 
     def _iterate(self, citations):
-        f = self.vars[self.getToken()[0].value()]
+        f = self.vars[self.get_token()[0].value()]
         for key in citations:
             self.current_entry_key = key
             self.current_entry = self.bib_data.entries[key]
@@ -240,14 +236,14 @@ class Interpreter(object):
         self.currentEntry = None
 
     def command_macro(self):
-        name = self.getToken()[0].value()
-        value = self.getToken()[0].value()
+        name = self.get_token()[0].value()
+        value = self.get_token()[0].value()
         self.macros[name] = value
 
     def expand_wildcard_citations(self):
         for citation in self.citations:
             if citation == '*':
-                for key in self.bib_data.entries.keys():
+                for key in self.bib_data.entries.iterkeys():
                     yield key
             else:
                 yield citation
@@ -255,7 +251,7 @@ class Interpreter(object):
     def command_read(self):
 #        print 'READ'
         p = self.bib_format.Parser(macros=self.macros, person_fields=[])
-        self.bib_data = p.parse_file(filename=self.bib_file)
+        self.bib_data = p.parse_files(self.bib_files)
         self.citations = list(self.expand_wildcard_citations())
 #        for k, v in self.bib_data.iteritems():
 #            print k
@@ -273,7 +269,7 @@ class Interpreter(object):
 
     def command_strings(self):
         #print 'STRINGS'
-        for id in self.getToken():
+        for id in self.get_token():
             self.vars[id.value()] = String()
 
     @staticmethod
