@@ -1,35 +1,31 @@
-# Copyright (C) 2006, 2007, 2008  Andrey Golovizin
-#
-# This file is part of pybtex.
-#
-# pybtex is free software; you can redistribute it and/or modify
-# under the terms of the GNU General Public License as published by the
-# Free Software Foundation; either version 2 of the License, or (at your
-# option) any later version.
-#
-# pybtex is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details.
-#
+# Copyright (C) 2006, 2007, 2008, 2009  Andrey Golovizin
+# 
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+# 
 # You should have received a copy of the GNU General Public License
-# along with pybtex; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
-# USA
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """BibTeX parser"""
 
 import codecs, locale
 from pyparsing import (
-        Word, CaselessLiteral, CharsNotIn,
+        Word, Literal, CaselessLiteral, CharsNotIn,
         nums, alphas, alphanums, printables, delimitedList, downcaseTokens,
         Suppress, Combine, Group, Dict,
         Forward, ZeroOrMore, Optional,
         ParseException,
 )
 from pybtex.core import Entry, Person
-from pybtex.database import BibliographyData
 from pybtex.database.input import ParserBase
+from pybtex.bibtex.utils import split_name_list
 
 month_names = {
     'jan': 'January',
@@ -48,46 +44,6 @@ month_names = {
 
 file_extension = 'bib'
 
-def split_name_list(s):
-    """
-    Split a list of names, separated by ' and '.
-
-    >>> split_name_list('Johnson and Peterson')
-    ['Johnson', 'Peterson']
-    >>> split_name_list('Armand and Peterson')
-    ['Armand', 'Peterson']
-    >>> split_name_list('Armand and anderssen')
-    ['Armand', 'anderssen']
-    >>> split_name_list('What a Strange{ }and Bizzare Name! and Peterson')
-    ['What a Strange{ }and Bizzare Name!', 'Peterson']
-    >>> split_name_list('What a Strange and{ }Bizzare Name! and Peterson')
-    ['What a Strange and{ }Bizzare Name!', 'Peterson']
-    """
-    after_space = False
-    brace_level = 0
-    name_start = 0
-    names = []
-    for pos, char in enumerate(s):
-        if char.isspace():
-            after_space = True
-        elif char == '{':
-            brace_level += 1
-            after_space = False
-        elif char == '}':
-            brace_level -= 1
-            after_space = False
-        elif (brace_level == 0
-                and after_space
-                and s[pos:pos + 3].lower() == 'and'
-                and s[pos + 3:pos+4].isspace()):
-            names.append(s[name_start:pos - 1])
-            name_start = pos + 4
-        else:
-            after_space = False
-    names.append(s[name_start:])
-    return names
-
-
 def join_lines(s, loc, toks):
     return [' '.join(tok.splitlines()) for tok in toks]
 
@@ -107,13 +63,15 @@ class Parser(ParserBase):
         at = Suppress('@')
         comma = Suppress(',')
 
-        quotedString = Combine(Suppress('"') + ZeroOrMore(CharsNotIn('"')).setParseAction(join_lines) + Suppress('"'))
-        bracedString = Forward()
-        bracedString << Combine(lbrace + ZeroOrMore(CharsNotIn('{}').setParseAction(join_lines) | bracedString) + rbrace)
+        innerBracedString = Forward()
+        innerBracedString << Combine(Literal('{') + ZeroOrMore(CharsNotIn('{}').setParseAction(join_lines) | innerBracedString) + Literal('}'))
+        quotedString = Combine(Suppress('"') + ZeroOrMore(CharsNotIn('"{').setParseAction(join_lines) | innerBracedString) + Suppress('"'))
+        bracedString = Combine(lbrace + ZeroOrMore(CharsNotIn('{}').setParseAction(join_lines) | innerBracedString) + rbrace)
         bibTeXString = quotedString | bracedString
 
-        macro_substitution = Word(alphanums).setParseAction(self.substitute_macro)
-        name = Word(alphanums + '!$&*+-./:;<>?[]^_`|').setParseAction(downcaseTokens)
+        name_chars = alphanums + '!$&*+-./:;<>?[\\]^_`|~\x7f'
+        macro_substitution = Word(name_chars).setParseAction(self.substitute_macro)
+        name = Word(name_chars).setParseAction(downcaseTokens)
         value = Combine(delimitedList(bibTeXString | Word(nums) | macro_substitution, delim='#'), adjacent=False)
 
         #fields
@@ -134,9 +92,9 @@ class Parser(ParserBase):
         entry_header = at + Word(alphas).setParseAction(downcaseTokens)
         entry_key = Word(printables.replace(',', ''))
         if kwargs.get('allow_keyless_entries', False):
-            entry_body = bibtexGroup(Optional(entry_key + comma, None) + Group(fields))
+            entry_body = bibtexGroup(Optional(entry_key + comma, None) + Group(fields) + Optional(comma))
         else:
-            entry_body = bibtexGroup(entry_key + comma + Group(fields))
+            entry_body = bibtexGroup(entry_key + comma + Group(fields) + Optional(comma))
         entry = entry_header + entry_body
         entry.setParseAction(self.process_entry)
 
@@ -174,11 +132,9 @@ class Parser(ParserBase):
         s = codecs.getreader(self.encoding)(stream).read()
 
         self.macros = dict(self.default_macros)
-        self.data = BibliographyData()
         try:
 #            entries = dict(entry[0][0] for entry in self.BibTeX_entry.scanString(s))
             self.BibTeX_entry.searchString(s)
-            return self.data
         except ParseException, e:
             print "%s: syntax error:" % getattr(stream, 'name', '<NO FILE>')
             print e
