@@ -1,21 +1,17 @@
-# Copyright (C) 2006, 2007, 2008  Andrey Golovizin
-#
-# This file is part of pybtex.
-#
-# pybtex is free software; you can redistribute it and/or modify
-# under the terms of the GNU General Public License as published by the
-# Free Software Foundation; either version 2 of the License, or (at your
-# option) any later version.
-#
-# pybtex is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details.
-#
+# Copyright (C) 2006, 2007, 2008, 2009  Andrey Golovizin
+# 
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+# 
 # You should have received a copy of the GNU General Public License
-# along with pybtex; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
-# USA
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 r"""(simple but) rich text formatting tools
 
@@ -27,8 +23,7 @@ Usage:
 this is a \emph{very} rich text
 >>> print t.plaintext()
 this is a very rich text
->>> t.capfirst()
->>> t.add_period()
+>>> t = t.capfirst().add_period()
 >>> print t.render(backend)
 This is a \emph{very} rich text.
 >>> print t.plaintext()
@@ -41,7 +36,7 @@ Some \emph{nested \texttt{Text objects}}.
 >>> print t.plaintext()
 Some nested Text objects.
 >>> from string import upper, lower
->>> t.apply(upper)
+>>> t = t.map(upper)
 >>> print t.render(backend)
 SOME \emph{NESTED \texttt{TEXT OBJECTS}}.
 >>> print t.plaintext()
@@ -61,6 +56,7 @@ one<nbsp>two<nbsp>three
 
 from copy import deepcopy
 from pybtex import textutils
+import string
 
 class Text(list):
     """
@@ -77,6 +73,27 @@ class Text(list):
 
         list.__init__(self, parts)
 
+    def __len__(self):
+        return sum(len(part) for part in self)
+
+    def __add__(self, other):
+        """Concatenate this Text with another Text or string.
+        >>> t = Text('a')
+        >>> print (t + 'b').plaintext()
+        ab
+        >>> print (t + t).plaintext()
+        aa
+        >>> print t.plaintext()
+        a
+        """
+
+        if isinstance(other, basestring):
+            other = [other]
+        return self.from_list(super(Text, self).__add__(other))
+
+    def from_list(self, lst):
+        return Text(*lst)
+
     def append(self, item):
         """Appends some text or something.
         Empty strings and similar things are ignored.
@@ -115,28 +132,62 @@ class Text(list):
             except AttributeError:
                 yield self, n
 
-    def apply(self, f):
-        """Apply a function to each part of the text."""  
+    def map(self, f, condition=None):
+        if condition is None:
+            condition = lambda index, length: True
+        def iter_map_with_condition():
+            length = len(self)
+            for index, child in enumerate(self):
+                if hasattr(child, 'map'):
+                    yield child.map(f, condition) if condition(index, length) else child
+                else:
+                    yield f(child) if condition(index, length) else child
+        return self.from_list(iter_map_with_condition())
 
-        for l, i in self.enumerate():
-            l[i] = f(l[i])
+    def upper(self):
+        return self.map(string.upper)
+
+    def upper_end(self):
+        return self.change_end(string.upper)
 
     def apply_to_start(self, f):
         """Apply a function to the last part of the text"""
-
-        l, i = self.enumerate().next()
-        l[i] = f(l[i])
+        return self.map(f, lambda index, length: index == 0)
 
     def apply_to_end(self, f):
         """Apply a function to the last part of the text"""
+        return self.map(f, lambda index, length: index == length - 1)
 
-        l, i = self.reversed().next()
-        l[i] = f(l[i])
+    def get_beginning(self):
+        try:
+            l, i = self.enumerate().next()
+        except StopIteration:
+            pass
+        else:
+            return l[i]
+
+    def get_end(self):
+        try:
+            l, i = self.reversed().next()
+        except StopIteration:
+            pass
+        else:
+            return l[i]
 
     def join(self, parts):
-        """Join a list using this text (like string.join)"""
+        """Join a list using this text (like string.join)
+
+        >>> print Text(' ').join([]).plaintext()
+        <BLANKLINE>
+        >>> print Text(' ').join(['a', 'b', 'c']).plaintext()
+        a b c
+        >>> print Text(nbsp).join(['a', 'b', 'c']).plaintext()
+        a<nbsp>b<nbsp>c
+        """
 
         joined = Text()
+        if not parts:
+            return joined
         for part in parts[:-1]:
             joined.extend([part, deepcopy(self)])
         joined.append(parts[-1])
@@ -147,13 +198,46 @@ class Text(list):
 
     def capfirst(self):
         """Capitalize the first letter of the text"""
+        return self.apply_to_start(textutils.capfirst)
 
-        self.apply_to_start(textutils.capfirst)
+    def add_period(self, period='.'):
+        """Add a period to the end of text, if necessary.
 
-    def add_period(self):
-        """Add a period to the end of text, if necessary"""
+        >>> import pybtex.backends.html
+        >>> html = pybtex.backends.html.Writer()
 
-        self.apply_to_end(textutils.add_period)
+        >>> text = Text("That's all, folks")
+        >>> print text.add_period().plaintext()
+        That's all, folks.
+
+        >>> text = Tag('emph', Text("That's all, folks"))
+        >>> print text.add_period().render(html)
+        <em>That's all, folks.</em>
+        >>> print text.add_period().add_period().render(html)
+        <em>That's all, folks.</em>
+
+        >>> text = Text("That's all, ", Tag('emph', 'folks'))
+        >>> print text.add_period().render(html)
+        That's all, <em>folks</em>.
+        >>> print text.add_period().add_period().render(html)
+        That's all, <em>folks</em>.
+
+        >>> text = Text("That's all, ", Tag('emph', 'folks.'))
+        >>> print text.add_period().render(html)
+        That's all, <em>folks.</em>
+
+        >>> text = Text("That's all, ", Tag('emph', 'folks'))
+        >>> print text.add_period('!').render(html)
+        That's all, <em>folks</em>!
+        >>> print text.add_period('!').add_period('.').render(html)
+        That's all, <em>folks</em>!
+        """
+
+        end = self.get_end()
+        if end and not textutils.is_terminated(end):
+            return self + period
+        else:
+            return self
 
 class Tag(Text):
     """A tag is somethins like <foo>some text</foo> in HTML
@@ -168,9 +252,13 @@ class Tag(Text):
     <em>emphasized text</em>
     """
 
+    def from_list(self, lst):
+        return Tag(self.name, *lst)
+
     def __init__(self, name, *args):
         self.name = name
         Text.__init__(self, *args)
+
     def render(self, backend):
         text = super(Tag, self).render(backend)
         return backend.format_tag(self.name, text)
@@ -191,8 +279,16 @@ class Symbol(object):
     def __init__(self, name):
         self.name = name
 
+    def __len__(self):
+        return 1
+
+    def __repr__(self):
+        return "Symbol('%s')" % self.name
+
     def __unicode__(self):
         return u'<%s>' % self.name
 
     def render(self, backend):
         return backend.symbols[self.name]
+
+nbsp = Symbol('nbsp')
